$LOAD_PATH.unshift __dir__

require "ruval/version"

module Ruval
  class Error < StandardError; end
  # Your code goes here...
end

#!/usr/bin/env -S ruby -W0
# vim: ft=ruby:syntax=ruby

DEFAULT_NEWLINES = 1
KEYWORDS_REGEX = /\b(?:class|module|def|if|do|end)\b|[{}\[\]()]/

def prog
  File.basename $0
end

def help
  puts <<~e
    Usage: #{prog} <file> [option]
    Print the value of statements in a file

    Options:
      -h, --help    print this message
      -n N          line spacing between evaluations (non-negative value, default 1)
  e
  exit
end

class String
  def remove_comments(padding)
    # not a perfect regex but good enough to detect most comments. eg it wont work for lines like "fun('#not a comment')"
    self.gsub(/ *#.*/, '').chop.ljust(padding) + ' # '
  end
end

# true if line is a comment
def comment?(line)
  line.strip[0] == '#'
end

if ARGV.empty? || (%w(-h --help) & ARGV).length > 0
  help
end

file = ARGV.shift

opts = {}
if ARGV.include?('-n')
  _, opts[:line_spacing]= ARGV[ARGV.index('-n'),2]
  opts[:line_spacing] = opts[:line_spacing].to_i
end
opts[:line_spacing] ||= DEFAULT_NEWLINES

if opts[:line_spacing] < 0
  puts "line_spacing cannot be negative"
  exit
end

opts[:colorize] = ARGV.include?('-c') ? true : false

# get longest line
longest = 0
File.read(file).each_line do |l|
  next if(comment?(l) || l.strip.empty?)
  line_width = l.remove_comments(0).length
  longest = line_width > longest ? line_width : longest
end

def update_scope(line, line_stack, scope)
  keywords = line.scan(KEYWORDS_REGEX)
  return if keywords.empty? && scope.empty?
  line_stack << line
  keywords.each do |kw|
    unless %w"end } ] )".include?(kw)
      scope.push kw
    else
      scope.pop
    end
  end
end

line_stack = []
scope = []

File.read(file).each_line do |l|
  next if(comment?(l) || l.strip.empty?)
  update_scope(l, line_stack, scope)
  next unless scope.empty?
  l = if line_stack.empty?
        opts[:line_spacing] == 0 ? l.remove_comments(longest) : l
      else
        line_stack.slice!(0,line_stack.length).join
      end
  value = eval(l, TOPLEVEL_BINDING)
  value = if value.class == String
            "\"#{value}\""
          elsif value.class == Symbol
            ":#{value}"
          else
            value
          end
  color = opts[:colorize] ? "\e[32m" : "\e[m"
  puts l + "#{color}=> #{value}\e[m" + "\n" * opts[:line_spacing]
end

